package com.fs.live.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fs.common.core.domain.R;
import com.fs.common.exception.base.BaseException;
import com.fs.common.utils.DateUtils;
import com.fs.common.utils.StringUtils;
import com.fs.live.domain.Live;
import com.fs.live.domain.LiveData;
import com.fs.live.domain.LiveVideo;
import com.fs.live.mapper.LiveMapper;
import com.fs.live.service.ILiveDataService;
import com.fs.live.service.ILiveService;
import com.fs.live.service.ILiveVideoService;
import com.fs.system.domain.SysConfig;
import com.fs.system.service.ISysConfigService;
import com.qcloud.cos.utils.Md5Utils;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 直播Service业务层处理
 *
 * @author fs
 * @date 2025-01-17
 */
@Service
@AllArgsConstructor
public class LiveServiceImpl extends ServiceImpl<LiveMapper, Live> implements ILiveService
{
    private final LiveMapper liveMapper;
    private final ILiveVideoService liveVideoService;
    private final ILiveDataService liveDataService;

    @Autowired
    private ISysConfigService sysConfigService;

    /**
     * 查询直播
     *
     * @param liveId 直播主键
     * @return 直播
     */
    @Override
    public Live selectLiveByLiveId(Long liveId){
        Live byId = liveMapper.selectLiveByLiveId(liveId);
        List<LiveVideo> videos = liveVideoService.listByLiveId(liveId, 1);
        if(!videos.isEmpty()){
            LiveVideo liveVideo = videos.get(0);
            byId.setVideoUrl(liveVideo.getVideoUrl());
            byId.setDuration(liveVideo.getDuration());
        }
        return byId;
    }

    /**
     * 查询企业直播
     * @param liveId            直播ID
     * @param companyId         企业ID
     * @param companyUserId     企业用户ID
     * @return Live
     */
    @Override
    public Live selectLiveByLiveIdAndCompanyIdAndCompanyUserId(Long liveId, Long companyId, Long companyUserId){
        Live byId = liveMapper.selectLiveByLiveIdAndCompanyIdAndCompanyUserId(liveId, companyId, companyUserId);
        List<LiveVideo> videos = liveVideoService.listByLiveId(liveId, 1);
        if(!videos.isEmpty()){
            LiveVideo liveVideo = videos.get(0);
            byId.setVideoUrl(liveVideo.getVideoUrl());
            byId.setDuration(liveVideo.getDuration());
        }
        return byId;
    }

    /**
     * 查询直播列表
     *
     * @param live 直播
     * @return 直播
     */
    @Override
    public List<Live> selectLiveList(Live live)
    {
        return liveMapper.selectLiveList(live);
    }


    /**
     * 查询直播列表
     *
     * @param
     * @return 直播
     */
    @Override
    public List<Live> selectLiveToStudent(String liveName,String status,String startTime,String finishTime)
    {
        return liveMapper.selectLiveToStudent(liveName,status,startTime,finishTime);
    }

    /**
     * 新增直播
     *
     * @param live 直播
     * @return 结果
     */
    @Override
    public int insertLive(Live live){
        if(live.getLiveType() == 2 && StringUtils.isEmpty(live.getVideoUrl())) {
            throw new BaseException("录播必须上传视屏");
        }
        live.setCreateTime(DateUtils.getNowDate());
        boolean save = save(live);
        LiveVideo liveVideo = new LiveVideo();
        liveVideo.setLiveId(live.getLiveId());
        liveVideo.setVideoUrl(live.getVideoUrl());
        liveVideo.setVideoType(1);
        liveVideo.setDuration(live.getDuration());
        liveVideoService.save(liveVideo);
        //新增直播数据表
        LiveData liveData = new LiveData();
        liveData.setLiveId(live.getLiveId());
        liveDataService.insertLiveData(liveData);
        return save ? 1 : 0;
    }

    /**
     * 修改直播
     *
     * @param live 直播
     * @return 结果
     */
    @Override
    public int updateLive(Live live){
        live.setUpdateTime(DateUtils.getNowDate());
        List<LiveVideo> videos = liveVideoService.listByLiveId(live.getLiveId(), 1);
        if(!videos.isEmpty()){
            LiveVideo liveVideo = videos.get(0);
            liveVideo.setVideoUrl(live.getVideoUrl());
            liveVideo.setUpdateTime(DateUtils.getNowDate());
            liveVideo.setDuration(live.getDuration());
            liveVideoService.updateById(liveVideo);
        }
        return liveMapper.updateLive(live);
    }

    /**
     * 批量删除直播
     *
     * @param liveIds 需要删除的直播主键
     * @return 结果
     */
    @Override
    public int deleteLiveByLiveIds(Long[] liveIds){
        return liveMapper.deleteLiveByLiveIds(liveIds);
    }

    /**
     * 删除直播信息
     *
     * @param liveId 直播主键
     * @return 结果
     */
    @Override
    public int deleteLiveByLiveId(Long liveId)
    {
        return liveMapper.deleteLiveByLiveId(liveId);
    }

    @Override
    public List<Live> liveList() {
        return liveMapper.liveList();
    }

    @Override
    public Map<String, Object> createLiveRoom(Long liveId) {
        Map<String, Object> resultMap = new HashMap<>();
        Live live = this.selectLiveByLiveId(liveId);
        if (live == null || live.getStatus() != 1 || live.getIsShow() != 1 || live.getIsAudit() != 1) {
            resultMap.put("code", "500");
            resultMap.put("msg", "您未拥有直播权限");
            return resultMap;
        }
        SysConfig sysConfig = sysConfigService.selectConfigByConfigKey("living.config");
        Map<String, String> livingConfigMap = JSON.parseObject(sysConfig.getConfigValue(), Map.class);
        if (livingConfigMap == null || livingConfigMap.isEmpty()) {
            resultMap.put("code", "500");
            resultMap.put("msg", "缺失直播配置");
            return Collections.emptyMap();
        }
        String rtmpPushUrl = generateRtmpPushUrl(livingConfigMap.get("domain"), livingConfigMap.get("app"), liveId.toString());
        String flvPlayUrl = generateFlvPlayUrl(livingConfigMap.get("http"), livingConfigMap.get("app"), liveId.toString());
        live.setRtmpUrl(rtmpPushUrl);
        live.setFlvHlsUrl(flvPlayUrl);
        this.updateLive(live);
        resultMap.put("code", "200");
        return resultMap;
    }

    @Override
    public Map<String, Object> getLiveRoom(String liveId) {
        Map<String, Object> resultMap = new HashMap<>();
        Live live = this.selectLiveByLiveId(Long.valueOf(liveId));
        if (live == null) {
            resultMap.put("code", "500");
            resultMap.put("msg", "未查询到直播间");
            return resultMap;
        }
        resultMap.put("code", "200");
        resultMap.put("livingUrl", live.getFlvHlsUrl());
        return resultMap;
    }



    public static String generateRtmpPushUrl(String domain, String app, String liveId) {
        long timestamp = System.currentTimeMillis() / 1000; // 秒级时间戳
        String SECRET_KEY = liveId + timestamp;
        String sign = Md5Utils.md5Hex(liveId + SECRET_KEY + timestamp);
        return String.format("rtmp://%s/%s/%s?timestamp=%d&sign=%s", domain, app, liveId,timestamp,sign);
    }

    /**
     * 生成 FLV 播放地址（用于前端播放）
     */
    public static String generateFlvPlayUrl(String domain, String app, String liveId) {
        long timestamp = System.currentTimeMillis() / 1000;
        String SECRET_KEY = liveId + timestamp;
        String sign = Md5Utils.md5Hex(liveId + SECRET_KEY + timestamp);
        return String.format("http://%s/%s/%s.flv?timestamp=%d&sign=%s", domain, app, liveId, timestamp, sign);
    }
}
